import {ItemGroup} from "../game-model/inventory/ItemGroup";
import {StoreComponent} from "../../../../app/storage/StoreComponent";
import {AbstractItemGroup} from "../game-model/inventory/AbstractItemGroup";
import {ConcreteItemGroup} from "../game-model/inventory/ConcreteItemGroup";
import {ModelComponentType} from "../game-model/ModelComponentType";
import {SerializeConstants} from "./SerializeConstants";

export class ItemSerializer {

  private serializedItemgroups: StoreComponent[] = []
  private static ignoredKeys: string[] = ['type', 'unsaved', 'children', "itemgroup", "manuallyInheritedGroups", "hierarchyInheritedGroups", "items"]

  public serializeItemgroups(itemgroups: ItemGroup[]): StoreComponent[] {
    itemgroups.forEach(itemgroup => {
      this.serializeSingleItemgroupHierarchy(itemgroup)
    })
    return this.serializedItemgroups;
  }

  private serializeSingleItemgroupHierarchy(itemgroup: ItemGroup) {
    if(itemgroup instanceof AbstractItemGroup) {
      const storeComponent = this.serializeItemgroup(itemgroup)
      this.serializedItemgroups.push(storeComponent)
      itemgroup.children.forEach(child => this.serializeSingleItemgroupHierarchy(child));
    } else {
      const storeComponent = this.serializeItemgroup(itemgroup);
      this.serializedItemgroups.push(storeComponent)
    }
  }

  private serializeItemgroup(itemgroup: ItemGroup): StoreComponent {
    const componentType = ModelComponentType.ITEMGROUP;
    const fileName = ItemSerializer.computeItemgroupPath(itemgroup);

    const jsonString = JSON.stringify(itemgroup, (key, value) => {
      if(ItemSerializer.ignoredKeys.includes(key)) {
        return undefined
      } else {

        if(key === 'key') {
          return value.characteristicName
        } else {
          return value;
        }

      }
    }, SerializeConstants.JSON_INDENT);
    console.log(fileName)
    return new StoreComponent(jsonString, fileName, componentType)
  }

  private static computeItemgroupPath(itemgroup: ItemGroup): string {
    const itemgroupPath: string[] = [];
    itemgroupPath.push(itemgroup.componentName);
    itemgroupPath.push(itemgroup.componentName);

    while(itemgroup.parentGroup !== undefined) {
      itemgroupPath.unshift(itemgroup.parentGroup.componentName);
      itemgroup = itemgroup.parentGroup;
    }

    return itemgroupPath.join("/");
  }
}
